@@ -771,6 +771,13 @@ void QQuick3DObjectPrivate::addToDirtyList()
                 QQuick3DObjectPrivate::get(nextDirtyItem)->prevDirtyItem = &nextDirtyItem;
             prevDirtyItem = &sceneManager->dirtyNodes[dirtyListIdx];
             sceneManager->dirtyNodes[dirtyListIdx] = q;
+        } else if (QSSGRenderGraphObject::isExtension(type)) {
+            const auto dirtyListIdx = QQuick3DSceneManager::extensionListIndex(type);
+            nextDirtyItem = sceneManager->dirtyExtensions[dirtyListIdx];
+            if (nextDirtyItem)
+                QQuick3DObjectPrivate::get(nextDirtyItem)->prevDirtyItem = &nextDirtyItem;
+            prevDirtyItem = &sceneManager->dirtyExtensions[dirtyListIdx];
+            sceneManager->dirtyExtensions[dirtyListIdx] = q;
         } else {
             const auto dirtyListIdx = QQuick3DSceneManager::resourceListIndex(type);
             nextDirtyItem = sceneManager->dirtyResources[dirtyListIdx];
@@ -877,8 +884,14 @@ void QQuick3DObjectPrivate::refSceneManager(QQuick3DSceneManager &c)
         }
 
         // NOTE: Simple tracking for resources that are shared between scenes.
-        if (&c != sceneManager && QSSGRenderGraphObject::isResource(type))
+        if (&c != sceneManager) {
             sharedResource = true;
+            for (int ii = 0; ii < childItems.size(); ++ii) {
+                QQuick3DObject *child = childItems.at(ii);
+                QQuick3DObjectPrivate::get(child)->sharedResource = sharedResource;
+                QQuick3DObjectPrivate::refSceneManager(child, c);
+            }
+        }
 
         return; // Scene manager already set.
     }
@@ -891,9 +904,12 @@ void QQuick3DObjectPrivate::refSceneManager(QQuick3DSceneManager &c)
 
     if (!parentItem)
         sceneManager->parentlessItems.insert(q);
+    else
+        sharedResource = QQuick3DObjectPrivate::get(parentItem)->sharedResource;
 
     for (int ii = 0; ii < childItems.size(); ++ii) {
         QQuick3DObject *child = childItems.at(ii);
+        QQuick3DObjectPrivate::get(child)->sharedResource = sharedResource;
         QQuick3DObjectPrivate::refSceneManager(child, c);
     }
 
@@ -916,18 +932,19 @@ void QQuick3DObjectPrivate::derefSceneManager()
     if (sceneManager)
         sceneManager->dirtyBoundingBoxList.removeAll(q);
 
-    if (spatialNode)
-        sceneManager->cleanup(spatialNode);
-    if (!parentItem)
-        sceneManager->parentlessItems.remove(q);
-
-    spatialNode = nullptr;
-
     for (int ii = 0; ii < childItems.size(); ++ii) {
         QQuick3DObject *child = childItems.at(ii);
         QQuick3DObjectPrivate::derefSceneManager(child);
     }
 
+    if (!parentItem)
+        sceneManager->parentlessItems.remove(q);
+
+    if (spatialNode) {
+        sceneManager->cleanup(spatialNode);
+        spatialNode = nullptr;
+    }
+
     sceneManager = nullptr;
 
     dirty(Window);
